const gulp = require('gulp');
const babel = require('gulp-babel');
const autoprefixer = require('gulp-autoprefixer');
const less = require("gulp-less");
const cssnano = require("gulp-cssnano");
const through2 = require('through2');
const concat = require('gulp-concat');

const paths = {
  dest: {
    lib: 'lib', // commonjs 文件存放的目录名 - 本块关注
    esm: 'esm', // ES module 文件存放的目录名 - 暂时不关心
    dist: 'dist', // umd文件存放的目录名 - 暂时不关心
    css: 'css', //发布样式文件css 最终放的目录
    cssUploadPath: 'upload'
  },
  styles: 'components/**/*.less', // 样式文件路径 - 暂时不关心
  cssStyles: 'components/**/*.css',
  css: 'css/**/*.css', //需要合并的css 样式
  scripts: [
    'components/**/*.{ts,tsx}',
    '!components/**/demo/*.{ts,tsx}',
    '!components/**/__tests__/*.{ts,tsx}',
  ], // 脚本文件路径
	resource:{
		png: 'components/**/*.{png,jpg,gif,svg}',
	}
};

/**
 * 编译脚本文件
 * @param {string} babelEnv babel环境变量
 * @param {string} destDir 目标目录
 */
function compileScripts(babelEnv, destDir) {
  const { scripts } = paths;
  // 设置环境变量
  process.env.BABEL_ENV = babelEnv;
  return gulp
    .src(scripts)
    .pipe(
      babel({
        plugins: [
        ],
      }),
    )
    .pipe(
      through2.obj(function z(file, encoding, next) {
        this.push(file.clone());
        // 找到目标
        if (file.path.match(/(\/|\\)style(\/|\\)index\.js/)) {
          const content = file.contents.toString(encoding);
          file.contents = Buffer.from(cssInjection(content)); // 文件内容处理
          file.path = file.path.replace(/index\.js/, 'css.js'); // 文件重命名
          this.push(file); // 新增该文件
          next();
        } else {
          next();
        }
      }),
    ) // 使用gulp-babel处理
    .pipe(gulp.dest(destDir));
}

/**
 * 编译cjs
 */
function compileCJS() {
  const { dest } = paths;
  return compileScripts('cjs', dest.lib);
}

/**
 * 编译esm
 */
function compileESM() {
  const { dest } = paths;
  return compileScripts('esm', dest.esm);
}

// 串行执行编译脚本任务（cjs,esm） 避免环境变量影响
const buildScripts = gulp.series(compileCJS, compileESM);
/**
 *拷贝其它资源
 *
 */
function copyResource() {
	 return gulp
     .src(paths.resource.png)
     .pipe(gulp.dest(paths.dest.lib))
     .pipe(gulp.dest(paths.dest.esm));
}
/**
 * 拷贝less文件
 */
function copyLess() {
  return gulp
    .src(paths.styles)
    .pipe(gulp.dest(paths.dest.lib))
    .pipe(gulp.dest(paths.dest.esm));
}

/**
 * 拷贝css文件
 */
function copyCss() {
  return gulp.src(paths.cssStyles).pipe(gulp.dest(paths.dest.lib)).pipe(gulp.dest(paths.dest.esm));
}

/**
 * 生成css文件
 */
function less2css() {
  return gulp
    .src(paths.styles)
    .pipe(
      less({
        javascriptEnabled: true,
      }),
    ) // 处理less文件
    .pipe(autoprefixer()) // 根据browserslistrc增加前缀
    .pipe(cssnano({ zindex: false, reduceIdents: false })) // 压缩
    .pipe(gulp.dest(paths.dest.lib))
    .pipe(gulp.dest(paths.dest.esm))
    .pipe(gulp.dest(paths.dest.css));
}

/**
 * 当前组件样式 import './index.less' => import './index.css'
 * 依赖的其他组件样式 import '../test-comp/style' => import '../test-comp/style/css.js'
 * 依赖的其他组件样式 import '../test-comp/style/index.js' => import '../test-comp/style/css.js'
 * @param {string} content
 */
function cssInjection(content) {
  return content
    .replace(/\/style\/?'/g, "/style/css'")
    .replace(/\/style\/?"/g, '/style/css"')
    .replace(/\.less/g, '.css');
}

/**
  合并css 文件
*/
function cssConcat() {
  return gulp
    .src(paths.css)
    .pipe(concat('iot-ui.min.css'))
    .pipe(gulp.dest(paths.dest.css))
    .pipe(gulp.dest(paths.dest.cssUploadPath))
}

function copyForUpload() {
  return gulp.src(`${paths.dest.esm}/base/theme.css`).pipe(gulp.dest(paths.dest.cssUploadPath));
}

// 整体并行执行任务
// const build = gulp.parallel(buildScripts, copyLess, less2css, cssConcat);
const build = gulp.series(buildScripts, copyCss,copyLess, less2css, cssConcat,copyResource, copyForUpload);
exports.build = build;

exports.default = build;
